#!/usr/bin/env node
const fs = require('fs');
let md = require('markdown-it')();
let config = require('./config.json');

const cmdArgs = new Set(process.argv.slice(2, process.argv.length));

const startHtml5Head = '<!DOCTYPE html><html><head>';
const startHtml5Body = '</head><body>';
const endHtml5 = '</body></html>';

const refineValue = function (attribute) {
    const attributeList = attribute.split(',');
    return attributeList.map((element) => {
        return element.trim(' ')
    })
};

const getMetaData = function (data) {
    const text = data.split('---\n', 2)[1];
    let resultantElements = {};
    text.split('\n').forEach((elem) => {
        if (elem.includes(':')) {
            let attribute = elem.split(':');
            resultantElements[attribute[0]] = refineValue(attribute[1])
        }
    });
    return resultantElements
};

const getMultipleElements = function (rawData) {
    return rawData
};

const getDocumentBody = function (data) {
    const text = data.split('---\n', 3)[2];
    return md.render(text);
};

const getHeadTags = function (data) {
    const metaData = getMetaData(data);
    let result = '';
    if ('title' in metaData) {
        result += '<title>' + metaData.title + '</title>'
    }
    if ('css' in metaData) {
        for (let index = 0; index < metaData['css'].length; ++index) {
            result += '<link rel="stylesheet" href="' + metaData['css'][index] + '">';
        }
    }
    return result
};

const lang = 'en';

const outputMessage = function (message) {
    process.stdout.write(config['msg'][lang][message].join('\n') + '\n')
};

const getBeforeBodyEnd = function (data) {
    const metaData = getMetaData(data);
    let result = '';
    if ('js' in metaData) {
        for (let index = 0; index < metaData['js'].length; ++index) {
            result += '<script type="text/javascript" src="' + metaData['js'][index] + '"></script>';
        }
    }
    return result;
};

const markdownFiles = Array.from(cmdArgs).filter((element) => {
    return element.includes('.md')
});

if (cmdArgs.size <= 0) {
    outputMessage('no_args')
} else if (cmdArgs.has('--help') || cmdArgs.has('-h')) {
    outputMessage('help')
} else if (cmdArgs.has('--title') || cmdArgs.has('-t')) {
    fs.readFile(markdownFiles[0], 'utf8', function (err, data) {
        process.stdout.write(
            getMetaData(data)['title'][0]
        )
    })
} else if (cmdArgs.has('--description') || cmdArgs.has('-d')) {
    fs.readFile(markdownFiles[0], 'utf8', function (err, data) {
        process.stdout.write(
            getMetaData(data)['description'][0]
        )
    })
}
else {
    fs.readFile(markdownFiles[0], 'utf8', function (err, data) {
        process.stdout.write(
            (startHtml5Head + getHeadTags(data) + startHtml5Body + getDocumentBody(data) + getBeforeBodyEnd(data) + endHtml5)
                .replace('', '')
        )
    })
}
